@@ -2,7 +2,234 @@ use crate::Error;
 use std::convert::TryFrom;
 
 mod block;
-pub use block::Block;
+pub use block::*;
+
+mod item;
+pub use item::Item;
+
+#[derive(Clone, Copy, PartialEq, Eq, Debug)]
+#[repr(i32)]
+/// Represents what specific statistic id is being referenced.
+pub enum StatisticID {
+    /// This statistic references the block registry ids.
+    Block(Block),
+    /// This statistic references the item registry ids.
+    Item(Item),
+    /// This statistic references the entity registry ids.
+    Entity(EntityType),
+    /// This statistic references the custom statistic registry ids.
+    Custom(CustomStatistic)
+}
+
+#[derive(Clone, Copy, PartialEq, Eq, Debug)]
+#[repr(i32)]
+/// Categorizes statstics into smaller groups.
+pub enum StatisticCateogry {
+    /// How many times this player has mined ___.
+    Mined = 0,
+    /// How many times this player has crafted ___.
+    Crafted = 1,
+    /// How many times this player has used ___.
+    Used = 2,
+    /// How many times this player has broken ___.
+    Broken = 3,
+    /// How many times this player has picked up ___.
+    PickedUp = 4,
+    /// How many times this player has dropped ___.
+    Dropped = 5,
+    /// How many times this player has killed ___.
+    Killed = 6,
+    /// How many times this player has been killed by ___.
+    KilledBy = 7,
+    /// Custom statistic.
+    Custom = 8
+}
+
+impl TryFrom<crate::VarInt> for StatisticCateogry {
+    type Error = Error;
+    fn try_from(value: crate::VarInt) -> Result<Self, Self::Error> {
+        match value.value() {
+            x if x == Self::Mined as i32 => Ok(Self::Mined),
+            x if x == Self::Crafted as i32 => Ok(Self::Crafted),
+            x if x == Self::Used as i32 => Ok(Self::Used),
+            x if x == Self::Broken as i32 => Ok(Self::Broken),
+            x if x == Self::PickedUp as i32 => Ok(Self::PickedUp),
+            x if x == Self::Dropped as i32 => Ok(Self::Dropped),
+            x if x == Self::Killed as i32 => Ok(Self::Killed),
+            x if x == Self::KilledBy as i32 => Ok(Self::KilledBy),
+            x if x == Self::Custom as i32 => Ok(Self::Custom),
+            _ => Err(Error::EnumOutOfBound)
+        }
+    }
+}
+
+#[derive(Clone, Copy, PartialEq, Eq, Debug)]
+#[repr(i32)]
+/// Represents a custom statistic.
+pub enum CustomStatistic {
+    /// Amount of times this player has left the game.
+    LeaveGame = 0,
+    /// Amount of minutes this player has played the game.
+    PlayOneMinute = 1,
+    /// Amount of ticks since this player has died.
+    TimeSinceDeath = 2,
+    /// Amount of ticks since this player has sept.
+    TimeSinceRest = 3,
+    /// Amount of ticks this player has spent sneaking.
+    SneakTime = 4,
+    /// Hundreths of blocks this player has walked.
+    WalkOneCm = 5,
+    /// Hundreths of blocks this player has crouch walked.
+    CrouchOneCm = 6,
+    /// Hundreths of blocks this player has sprinted.
+    SprintOneCm = 7,
+    /// Hundreths of blocks this player has walked over water using frost walker boots.
+    WalkOnWaterOneCm = 8,
+    /// Hundreths of blocks this player has fallen.
+    FallOneCm = 9,
+    /// Hundreths of blocks this player has climbed.
+    ClimbOneCm = 10,
+    /// Hundreths of blocks this player has flown.
+    FlyOneCm = 11,
+    /// Hundreths of blocks this player has walked in water.
+    WalkUnderWaterOneCm = 12,
+    /// Hundreths of blocks this player has travelled in a minecart.
+    MinecartOneCm = 13,
+    /// Hundreths of blocks this player has rowwed in a boat.
+    BoatOneCm = 14,
+    /// Hundreths of blocks this player has ridden on a pig.
+    PigOneCm = 15,
+    /// Hundreths of blocks this player has ridden on a horse.
+    HorseOneCm = 16,
+    /// Hundreths of blocks this player has levitated.
+    AviateOneCm = 17,
+    /// Hundreths of blocks this player has swam.
+    SwimOneCm = 18,
+    /// Hundreths of blocks this player has ridden on a strider.
+    StriderOneCm = 19,
+    /// Number of times this player has jumped.
+    Jump = 20,
+    /// Number of times this player has dropped an item.
+    Drop = 21,
+    /// Tenths of hitpoints done by this player.
+    DamageDealt = 22,
+    /// Tenths of hitpoints done by this player that were absorbed.
+    DamageDealtAbsorbed = 23,
+    /// Tenths of hitpoints done by this player that were resisted.
+    DamageDealtResisted = 24,
+    /// Tenths of hitpoints of damage taken by this player.
+    DamageTaken = 25,
+    /// Tenths of hitpoints of damage this player has blocked with a shield.
+    DamageBlockedByShield = 26,
+    /// Tenths of hitpoints of damage this player has absorbed.
+    DamageAbsorbed = 27,
+    /// Tenths of hitpoints of damage this player has resisted.
+    DamageResisted = 28,
+    /// Number of times this player has died.
+    Deaths = 29,
+    /// Number of mobs this player has killed.
+    MobKills = 30,
+    /// Number of animals this player has bred.
+    AnimalsBred = 31,
+    /// Number of players this player has killed.
+    PlayerKills = 32,
+    /// Number of fish this player has caught.
+    FishCaught = 33,
+    /// Number of times this player has opened the villager GUI.
+    TalkedToVillager = 34,
+    /// Number of times this player has traded with a villager.
+    TradedWithVillager = 35,
+    /// Number of times this player has eaten a piece of cake.
+    EatCakeSlice = 36,
+    /// Number of times this player has filled a cauldron.
+    FillCauldron = 37,
+    /// Number of times this player has interacted with a cauldron.
+    UseCauldron = 38,
+    /// Number of times this player has removed dye from leather armor.
+    CleanArmor = 39,
+    /// Number of times this player has removed patterns from banners.
+    CleanBanner = 40,
+    /// Number of times this player has removed dye from shulker boxes.
+    CleanShulkerBox = 41,
+    /// Number of times this player has opened the brewing stand GUI.
+    InteractWithBrewingstand = 42,
+    /// Number of times this player has opened the beacon GUI.
+    InteractWithBeacon = 43,
+    /// Number of times this player has opened the dropper GUI.
+    InspectDropper = 44,
+    /// Number of times this player has opened the hopper GUI.
+    InspectHopper = 45,
+    /// Number of times this player has opened the dispenser GUI.
+    InspectDispenser = 46,
+    /// Number of times this player has played a noteblock.
+    PlayNoteblock = 47,
+    /// Number of times this player has changed the pitch of a noteblock.
+    TuneNoteblock = 48,
+    /// Number of times this player has planted something in a flowerpot.
+    PotFlower = 49,
+    /// Number of times this player has activated a trapped chest.
+    TriggerTrappedChest = 50,
+    /// Number of times this player has opened the enderchest GUI.
+    OpenEnderchest = 51,
+    /// Number of times this player has enchanted an item.
+    EnchantItem = 52,
+    /// Number of times this player has played a music disk in a jukebox.
+    PlayRecord = 53,
+    /// Number of times this player has opened the furnace GUI.
+    InteractWithFurnace = 54,
+    /// Number of times this player has opened the crafting table GUI.
+    InteractWithCraftingTable = 55,
+    /// Number of times this player has opened the chest GUI.
+    OpenChest = 56,
+    /// Number of times this player has slept in a bed.
+    SleepInBed = 57,
+    /// Number of times this player has opened the shulker box GUI.
+    OpenShulkerBox = 58,
+    /// Number of times this player has opened the barrel GUI.
+    OpenBarrel = 59,
+    /// Number of times this player has opened the blast furnace GUI.
+    InteractWithBlastFurnace = 60,
+    /// Number of times this player has opened the smoker GUI.
+    InteractWithSmoker = 61,
+    /// Number of times this player has placed or removed a book on a lectern.
+    InteractWithLectern = 62,
+    /// Number of times this player has placed or removed an item on a campfire.
+    InteractWithCampfire = 63,
+    /// Number of times this player has opened the cartography table GUI.
+    InteractWithCartographyTable = 64,
+    /// Number of times this player has opened the loom GUI.
+    InteractWithLoom = 65,
+    /// Number of times this player has opened the stonecutter GUI.
+    InteractWithStonecutter = 66,
+    /// Number of times this player has rung a bell.
+    BellRing = 67,
+    /// Number of times this player has caused a raid.
+    RaidTrigger = 68,
+    /// Number of times this player has helped defeat a raid.
+    RaidWin = 69,
+    /// Number of times this player has opened the anvil GUI.
+    InteractWithAnvil = 70,
+    /// Number of times this player has opened the grindstone GUI.
+    InteractWithGrindstone = 71,
+    /// Number of times this player has successfully hit a target block.
+    TargetHit = 72,
+    /// Number of times this player has opened th smithing table GUI.
+    InteractWithSmithingTable = 73
+}
+
+impl TryFrom<crate::VarInt> for CustomStatistic {
+    type Error = Error;
+    fn try_from(value: crate::VarInt) -> Result<Self, Self::Error> {
+        match value.value() {
+            x if x == Self::LeaveGame as i32 => Ok(Self::LeaveGame),
+            x if x == Self::PlayOneMinute as i32 => Ok(Self::PlayOneMinute),
+            x if x == Self::TimeSinceDeath as i32 => Ok(Self::TimeSinceDeath),
+            x if x == Self::TimeSinceRest as i32 => Ok(Self::TimeSinceRest),
+            x if x == Self::SneakTime as i32 => Ok(Self::SneakTime),
+            _ => Err(Error::EnumOutOfBound)
+        }
+    }
+}
 
 #[derive(Clone, Copy, PartialEq, Eq, Debug)]
 #[repr(i32)]
@@ -15,11 +242,10 @@ pub enum DiggingStatus {
 impl TryFrom<crate::VarInt> for DiggingStatus {
     type Error = Error;
     fn try_from(value: crate::VarInt) -> Result<Self, Self::Error> {
-        use crate::VarInt;
-        match value {
-            x if x == VarInt::from_value(Self::StartDigging as i32)? => Ok(Self::StartDigging),
-            x if x == VarInt::from_value(Self::CancelDigging as i32)? => Ok(Self::CancelDigging),
-            x if x == VarInt::from_value(Self::FinishDigging as i32)? => Ok(Self::FinishDigging),
+        match value.value() {
+            x if x == Self::StartDigging as i32 => Ok(Self::StartDigging),
+            x if x == Self::CancelDigging as i32 => Ok(Self::CancelDigging),
+            x if x == Self::FinishDigging as i32 => Ok(Self::FinishDigging),
             _ => Err(Error::EnumOutOfBound)
         }
     }
@@ -95,34 +321,33 @@ pub enum PaintingType {
 impl TryFrom<crate::VarInt> for PaintingType {
     type Error = Error;
     fn try_from(value: crate::VarInt) -> Result<Self, Self::Error> {
-        use crate::VarInt;
-        match value {
-            x if x == VarInt::from_value(Self::Kebab as i32)? => Ok(Self::Kebab),
-            x if x == VarInt::from_value(Self::Aztec as i32)? => Ok(Self::Aztec),
-            x if x == VarInt::from_value(Self::Alban as i32)? => Ok(Self::Alban),
-            x if x == VarInt::from_value(Self::Aztec2 as i32)? => Ok(Self::Aztec2),
-            x if x == VarInt::from_value(Self::Bomb as i32)? => Ok(Self::Bomb),
-            x if x == VarInt::from_value(Self::Plant as i32)? => Ok(Self::Plant),
-            x if x == VarInt::from_value(Self::Wasteland as i32)? => Ok(Self::Wasteland),
-            x if x == VarInt::from_value(Self::Pool as i32)? => Ok(Self::Pool),
-            x if x == VarInt::from_value(Self::Courbert as i32)? => Ok(Self::Courbert),
-            x if x == VarInt::from_value(Self::Sea as i32)? => Ok(Self::Sea),
-            x if x == VarInt::from_value(Self::Sunset as i32)? => Ok(Self::Sunset),
-            x if x == VarInt::from_value(Self::Creebet as i32)? => Ok(Self::Creebet),
-            x if x == VarInt::from_value(Self::Wanderer as i32)? => Ok(Self::Wanderer),
-            x if x == VarInt::from_value(Self::Graham as i32)? => Ok(Self::Graham),
-            x if x == VarInt::from_value(Self::Match as i32)? => Ok(Self::Match),
-            x if x == VarInt::from_value(Self::Bust as i32)? => Ok(Self::Bust),
-            x if x == VarInt::from_value(Self::Stage as i32)? => Ok(Self::Stage),
-            x if x == VarInt::from_value(Self::Void as i32)? => Ok(Self::Void),
-            x if x == VarInt::from_value(Self::SkullAndRoses as i32)? => Ok(Self::SkullAndRoses),
-            x if x == VarInt::from_value(Self::Wither as i32)? => Ok(Self::Wither),
-            x if x == VarInt::from_value(Self::Fighters as i32)? => Ok(Self::Fighters),
-            x if x == VarInt::from_value(Self::Pointer as i32)? => Ok(Self::Pointer),
-            x if x == VarInt::from_value(Self::Pigscene as i32)? => Ok(Self::Pigscene),
-            x if x == VarInt::from_value(Self::BurningSkull as i32)? => Ok(Self::BurningSkull),
-            x if x == VarInt::from_value(Self::Skeleton as i32)? => Ok(Self::Skeleton),
-            x if x == VarInt::from_value(Self::DonkeyKong as i32)? => Ok(Self::DonkeyKong),
+        match value.value() {
+            x if x == Self::Kebab as i32 => Ok(Self::Kebab),
+            x if x == Self::Aztec as i32 => Ok(Self::Aztec),
+            x if x == Self::Alban as i32 => Ok(Self::Alban),
+            x if x == Self::Aztec2 as i32 => Ok(Self::Aztec2),
+            x if x == Self::Bomb as i32 => Ok(Self::Bomb),
+            x if x == Self::Plant as i32 => Ok(Self::Plant),
+            x if x == Self::Wasteland as i32 => Ok(Self::Wasteland),
+            x if x == Self::Pool as i32 => Ok(Self::Pool),
+            x if x == Self::Courbert as i32 => Ok(Self::Courbert),
+            x if x == Self::Sea as i32 => Ok(Self::Sea),
+            x if x == Self::Sunset as i32 => Ok(Self::Sunset),
+            x if x == Self::Creebet as i32 => Ok(Self::Creebet),
+            x if x == Self::Wanderer as i32 => Ok(Self::Wanderer),
+            x if x == Self::Graham as i32 => Ok(Self::Graham),
+            x if x == Self::Match as i32 => Ok(Self::Match),
+            x if x == Self::Bust as i32 => Ok(Self::Bust),
+            x if x == Self::Stage as i32 => Ok(Self::Stage),
+            x if x == Self::Void as i32 => Ok(Self::Void),
+            x if x == Self::SkullAndRoses as i32 => Ok(Self::SkullAndRoses),
+            x if x == Self::Wither as i32 => Ok(Self::Wither),
+            x if x == Self::Fighters as i32 => Ok(Self::Fighters),
+            x if x == Self::Pointer as i32 => Ok(Self::Pointer),
+            x if x == Self::Pigscene as i32 => Ok(Self::Pigscene),
+            x if x == Self::BurningSkull as i32 => Ok(Self::BurningSkull),
+            x if x == Self::Skeleton as i32 => Ok(Self::Skeleton),
+            x if x == Self::DonkeyKong as i32 => Ok(Self::DonkeyKong),
             _ => Err(Error::EnumOutOfBound)
         }
     }
@@ -305,121 +530,120 @@ pub enum EntityType {
 impl TryFrom<crate::VarInt> for EntityType {
     type Error = Error;
     fn try_from(value: crate::VarInt) -> Result<Self, Self::Error> {
-        use crate::VarInt;
-        match value {
-            x if x == VarInt::from_value(Self::AreaEffectCloud as i32)? => Ok(Self::AreaEffectCloud),
-            x if x == VarInt::from_value(Self::ArmorStand as i32)? => Ok(Self::ArmorStand),
-            x if x == VarInt::from_value(Self::Arrow as i32)? => Ok(Self::Arrow),
-            x if x == VarInt::from_value(Self::Axolotl as i32)? => Ok(Self::Axolotl),
-            x if x == VarInt::from_value(Self::Bat as i32)? => Ok(Self::Bat),
-            x if x == VarInt::from_value(Self::Bee as i32)? => Ok(Self::Bee),
-            x if x == VarInt::from_value(Self::Blaze as i32)? => Ok(Self::Blaze),
-            x if x == VarInt::from_value(Self::Boat as i32)? => Ok(Self::Boat),
-            x if x == VarInt::from_value(Self::Cat as i32)? => Ok(Self::Cat),
-            x if x == VarInt::from_value(Self::CaveSpider as i32)? => Ok(Self::CaveSpider),
-            x if x == VarInt::from_value(Self::Chicken as i32)? => Ok(Self::Chicken),
-            x if x == VarInt::from_value(Self::Cod as i32)? => Ok(Self::Cod),
-            x if x == VarInt::from_value(Self::Cow as i32)? => Ok(Self::Cow),
-            x if x == VarInt::from_value(Self::Creeper as i32)? => Ok(Self::Creeper),
-            x if x == VarInt::from_value(Self::Dolphin as i32)? => Ok(Self::Dolphin),
-            x if x == VarInt::from_value(Self::Donkey as i32)? => Ok(Self::Donkey),
-            x if x == VarInt::from_value(Self::DragonFireball as i32)? => Ok(Self::DragonFireball),
-            x if x == VarInt::from_value(Self::Drowned as i32)? => Ok(Self::Drowned),
-            x if x == VarInt::from_value(Self::ElderGuardian as i32)? => Ok(Self::ElderGuardian),
-            x if x == VarInt::from_value(Self::EndCrystal as i32)? => Ok(Self::EndCrystal),
-            x if x == VarInt::from_value(Self::EnderDragon as i32)? => Ok(Self::EnderDragon),
-            x if x == VarInt::from_value(Self::Enderman as i32)? => Ok(Self::Enderman),
-            x if x == VarInt::from_value(Self::Endermite as i32)? => Ok(Self::Endermite),
-            x if x == VarInt::from_value(Self::Evoker as i32)? => Ok(Self::Evoker),
-            x if x == VarInt::from_value(Self::EvokerFangs as i32)? => Ok(Self::EvokerFangs),
-            x if x == VarInt::from_value(Self::ExperienceOrb as i32)? => Ok(Self::ExperienceOrb),
-            x if x == VarInt::from_value(Self::EyeOfEnder as i32)? => Ok(Self::EyeOfEnder),
-            x if x == VarInt::from_value(Self::FallingBlock as i32)? => Ok(Self::FallingBlock),
-            x if x == VarInt::from_value(Self::FireworkRocket as i32)? => Ok(Self::FireworkRocket),
-            x if x == VarInt::from_value(Self::Fox as i32)? => Ok(Self::Fox),
-            x if x == VarInt::from_value(Self::Ghast as i32)? => Ok(Self::Ghast),
-            x if x == VarInt::from_value(Self::Giant as i32)? => Ok(Self::Giant),
-            x if x == VarInt::from_value(Self::GlowItemFrame as i32)? => Ok(Self::GlowItemFrame),
-            x if x == VarInt::from_value(Self::GlowSquid as i32)? => Ok(Self::GlowSquid),
-            x if x == VarInt::from_value(Self::Goat as i32)? => Ok(Self::Goat),
-            x if x == VarInt::from_value(Self::Guardian as i32)? => Ok(Self::Guardian),
-            x if x == VarInt::from_value(Self::Hoglin as i32)? => Ok(Self::Hoglin),
-            x if x == VarInt::from_value(Self::Horse as i32)? => Ok(Self::Horse),
-            x if x == VarInt::from_value(Self::Husk as i32)? => Ok(Self::Husk),
-            x if x == VarInt::from_value(Self::Illusioner as i32)? => Ok(Self::Illusioner),
-            x if x == VarInt::from_value(Self::IronGolem as i32)? => Ok(Self::IronGolem),
-            x if x == VarInt::from_value(Self::Item as i32)? => Ok(Self::Item),
-            x if x == VarInt::from_value(Self::ItemFrame as i32)? => Ok(Self::ItemFrame),
-            x if x == VarInt::from_value(Self::Fireball as i32)? => Ok(Self::Fireball),
-            x if x == VarInt::from_value(Self::LeashKnot as i32)? => Ok(Self::LeashKnot),
-            x if x == VarInt::from_value(Self::LightningBolt as i32)? => Ok(Self::LightningBolt),
-            x if x == VarInt::from_value(Self::Llama as i32)? => Ok(Self::Llama),
-            x if x == VarInt::from_value(Self::LlamaSpit as i32)? => Ok(Self::LlamaSpit),
-            x if x == VarInt::from_value(Self::MagmaCube as i32)? => Ok(Self::MagmaCube),
-            x if x == VarInt::from_value(Self::Marker as i32)? => Ok(Self::Marker),
-            x if x == VarInt::from_value(Self::Minecart as i32)? => Ok(Self::Minecart),
-            x if x == VarInt::from_value(Self::MinecartChest as i32)? => Ok(Self::MinecartChest),
-            x if x == VarInt::from_value(Self::MinecartCommandBlock as i32)? => Ok(Self::MinecartCommandBlock),
-            x if x == VarInt::from_value(Self::MinecartFurnace as i32)? => Ok(Self::MinecartFurnace),
-            x if x == VarInt::from_value(Self::MinecartHopper as i32)? => Ok(Self::MinecartHopper),
-            x if x == VarInt::from_value(Self::MinecartSpawner as i32)? => Ok(Self::MinecartSpawner),
-            x if x == VarInt::from_value(Self::MinecartTNT as i32)? => Ok(Self::MinecartTNT),
-            x if x == VarInt::from_value(Self::Mule as i32)? => Ok(Self::Mule),
-            x if x == VarInt::from_value(Self::Mooshroom as i32)? => Ok(Self::Mooshroom),
-            x if x == VarInt::from_value(Self::Ocelot as i32)? => Ok(Self::Ocelot),
-            x if x == VarInt::from_value(Self::Painting as i32)? => Ok(Self::Painting),
-            x if x == VarInt::from_value(Self::Panda as i32)? => Ok(Self::Panda),
-            x if x == VarInt::from_value(Self::Parrot as i32)? => Ok(Self::Parrot),
-            x if x == VarInt::from_value(Self::Phantom as i32)? => Ok(Self::Phantom),
-            x if x == VarInt::from_value(Self::Pig as i32)? => Ok(Self::Pig),
-            x if x == VarInt::from_value(Self::Piglin as i32)? => Ok(Self::Piglin),
-            x if x == VarInt::from_value(Self::PiglinBrute as i32)? => Ok(Self::PiglinBrute),
-            x if x == VarInt::from_value(Self::Pillager as i32)? => Ok(Self::Pillager),
-            x if x == VarInt::from_value(Self::PolarBear as i32)? => Ok(Self::PolarBear),
-            x if x == VarInt::from_value(Self::PrimedTNT as i32)? => Ok(Self::PrimedTNT),
-            x if x == VarInt::from_value(Self::Pufferfish as i32)? => Ok(Self::Pufferfish),
-            x if x == VarInt::from_value(Self::Rabbit as i32)? => Ok(Self::Rabbit),
-            x if x == VarInt::from_value(Self::Ravager as i32)? => Ok(Self::Ravager),
-            x if x == VarInt::from_value(Self::Salmon as i32)? => Ok(Self::Salmon),
-            x if x == VarInt::from_value(Self::Sheep as i32)? => Ok(Self::Sheep),
-            x if x == VarInt::from_value(Self::Shulker as i32)? => Ok(Self::Shulker),
-            x if x == VarInt::from_value(Self::ShulkerBullet as i32)? => Ok(Self::ShulkerBullet),
-            x if x == VarInt::from_value(Self::Silverfish as i32)? => Ok(Self::Silverfish),
-            x if x == VarInt::from_value(Self::Skeleton as i32)? => Ok(Self::Skeleton),
-            x if x == VarInt::from_value(Self::SkeletonHorse as i32)? => Ok(Self::SkeletonHorse),
-            x if x == VarInt::from_value(Self::Slime as i32)? => Ok(Self::Slime),
-            x if x == VarInt::from_value(Self::SmallFireball as i32)? => Ok(Self::SmallFireball),
-            x if x == VarInt::from_value(Self::SnowGolem as i32)? => Ok(Self::SnowGolem),
-            x if x == VarInt::from_value(Self::Snowball as i32)? => Ok(Self::Snowball),
-            x if x == VarInt::from_value(Self::SpectralArrow as i32)? => Ok(Self::SpectralArrow),
-            x if x == VarInt::from_value(Self::Spider as i32)? => Ok(Self::Spider),
-            x if x == VarInt::from_value(Self::Squid as i32)? => Ok(Self::Squid),
-            x if x == VarInt::from_value(Self::Stray as i32)? => Ok(Self::Stray),
-            x if x == VarInt::from_value(Self::Strider as i32)? => Ok(Self::Strider),
-            x if x == VarInt::from_value(Self::ThrownEgg as i32)? => Ok(Self::ThrownEgg),
-            x if x == VarInt::from_value(Self::ThrownEnderPearl as i32)? => Ok(Self::ThrownEnderPearl),
-            x if x == VarInt::from_value(Self::ThrownExperienceBottle as i32)? => Ok(Self::ThrownExperienceBottle),
-            x if x == VarInt::from_value(Self::ThrownPotion as i32)? => Ok(Self::ThrownPotion),
-            x if x == VarInt::from_value(Self::ThrownTrident as i32)? => Ok(Self::ThrownTrident),
-            x if x == VarInt::from_value(Self::TraderLlama as i32)? => Ok(Self::TraderLlama),
-            x if x == VarInt::from_value(Self::TropicalFish as i32)? => Ok(Self::TropicalFish),
-            x if x == VarInt::from_value(Self::Turtle as i32)? => Ok(Self::Turtle),
-            x if x == VarInt::from_value(Self::Vex as i32)? => Ok(Self::Vex),
-            x if x == VarInt::from_value(Self::Villager as i32)? => Ok(Self::Villager),
-            x if x == VarInt::from_value(Self::Vindicator as i32)? => Ok(Self::Vindicator),
-            x if x == VarInt::from_value(Self::WanderingTrader as i32)? => Ok(Self::WanderingTrader),
-            x if x == VarInt::from_value(Self::Witch as i32)? => Ok(Self::Witch),
-            x if x == VarInt::from_value(Self::Wither as i32)? => Ok(Self::Wither),
-            x if x == VarInt::from_value(Self::WitherSkeleton as i32)? => Ok(Self::WitherSkeleton),
-            x if x == VarInt::from_value(Self::WitherSkull as i32)? => Ok(Self::WitherSkull),
-            x if x == VarInt::from_value(Self::Wolf as i32)? => Ok(Self::Wolf),
-            x if x == VarInt::from_value(Self::Zoglin as i32)? => Ok(Self::Zoglin),
-            x if x == VarInt::from_value(Self::Zombie as i32)? => Ok(Self::Zombie),
-            x if x == VarInt::from_value(Self::ZombieHorse as i32)? => Ok(Self::ZombieHorse),
-            x if x == VarInt::from_value(Self::ZombieVillager as i32)? => Ok(Self::ZombieVillager),
-            x if x == VarInt::from_value(Self::ZombifiedPiglin as i32)? => Ok(Self::ZombifiedPiglin),
-            x if x == VarInt::from_value(Self::Player as i32)? => Ok(Self::Player),
-            x if x == VarInt::from_value(Self::FishingHook as i32)? => Ok(Self::FishingHook),
+        match value.value() {
+            x if x == Self::AreaEffectCloud as i32 => Ok(Self::AreaEffectCloud),
+            x if x == Self::ArmorStand as i32 => Ok(Self::ArmorStand),
+            x if x == Self::Arrow as i32 => Ok(Self::Arrow),
+            x if x == Self::Axolotl as i32 => Ok(Self::Axolotl),
+            x if x == Self::Bat as i32 => Ok(Self::Bat),
+            x if x == Self::Bee as i32 => Ok(Self::Bee),
+            x if x == Self::Blaze as i32 => Ok(Self::Blaze),
+            x if x == Self::Boat as i32 => Ok(Self::Boat),
+            x if x == Self::Cat as i32 => Ok(Self::Cat),
+            x if x == Self::CaveSpider as i32 => Ok(Self::CaveSpider),
+            x if x == Self::Chicken as i32 => Ok(Self::Chicken),
+            x if x == Self::Cod as i32 => Ok(Self::Cod),
+            x if x == Self::Cow as i32 => Ok(Self::Cow),
+            x if x == Self::Creeper as i32 => Ok(Self::Creeper),
+            x if x == Self::Dolphin as i32 => Ok(Self::Dolphin),
+            x if x == Self::Donkey as i32 => Ok(Self::Donkey),
+            x if x == Self::DragonFireball as i32 => Ok(Self::DragonFireball),
+            x if x == Self::Drowned as i32 => Ok(Self::Drowned),
+            x if x == Self::ElderGuardian as i32 => Ok(Self::ElderGuardian),
+            x if x == Self::EndCrystal as i32 => Ok(Self::EndCrystal),
+            x if x == Self::EnderDragon as i32 => Ok(Self::EnderDragon),
+            x if x == Self::Enderman as i32 => Ok(Self::Enderman),
+            x if x == Self::Endermite as i32 => Ok(Self::Endermite),
+            x if x == Self::Evoker as i32 => Ok(Self::Evoker),
+            x if x == Self::EvokerFangs as i32 => Ok(Self::EvokerFangs),
+            x if x == Self::ExperienceOrb as i32 => Ok(Self::ExperienceOrb),
+            x if x == Self::EyeOfEnder as i32 => Ok(Self::EyeOfEnder),
+            x if x == Self::FallingBlock as i32 => Ok(Self::FallingBlock),
+            x if x == Self::FireworkRocket as i32 => Ok(Self::FireworkRocket),
+            x if x == Self::Fox as i32 => Ok(Self::Fox),
+            x if x == Self::Ghast as i32 => Ok(Self::Ghast),
+            x if x == Self::Giant as i32 => Ok(Self::Giant),
+            x if x == Self::GlowItemFrame as i32 => Ok(Self::GlowItemFrame),
+            x if x == Self::GlowSquid as i32 => Ok(Self::GlowSquid),
+            x if x == Self::Goat as i32 => Ok(Self::Goat),
+            x if x == Self::Guardian as i32 => Ok(Self::Guardian),
+            x if x == Self::Hoglin as i32 => Ok(Self::Hoglin),
+            x if x == Self::Horse as i32 => Ok(Self::Horse),
+            x if x == Self::Husk as i32 => Ok(Self::Husk),
+            x if x == Self::Illusioner as i32 => Ok(Self::Illusioner),
+            x if x == Self::IronGolem as i32 => Ok(Self::IronGolem),
+            x if x == Self::Item as i32 => Ok(Self::Item),
+            x if x == Self::ItemFrame as i32 => Ok(Self::ItemFrame),
+            x if x == Self::Fireball as i32 => Ok(Self::Fireball),
+            x if x == Self::LeashKnot as i32 => Ok(Self::LeashKnot),
+            x if x == Self::LightningBolt as i32 => Ok(Self::LightningBolt),
+            x if x == Self::Llama as i32 => Ok(Self::Llama),
+            x if x == Self::LlamaSpit as i32 => Ok(Self::LlamaSpit),
+            x if x == Self::MagmaCube as i32 => Ok(Self::MagmaCube),
+            x if x == Self::Marker as i32 => Ok(Self::Marker),
+            x if x == Self::Minecart as i32 => Ok(Self::Minecart),
+            x if x == Self::MinecartChest as i32 => Ok(Self::MinecartChest),
+            x if x == Self::MinecartCommandBlock as i32 => Ok(Self::MinecartCommandBlock),
+            x if x == Self::MinecartFurnace as i32 => Ok(Self::MinecartFurnace),
+            x if x == Self::MinecartHopper as i32 => Ok(Self::MinecartHopper),
+            x if x == Self::MinecartSpawner as i32 => Ok(Self::MinecartSpawner),
+            x if x == Self::MinecartTNT as i32 => Ok(Self::MinecartTNT),
+            x if x == Self::Mule as i32 => Ok(Self::Mule),
+            x if x == Self::Mooshroom as i32 => Ok(Self::Mooshroom),
+            x if x == Self::Ocelot as i32 => Ok(Self::Ocelot),
+            x if x == Self::Painting as i32 => Ok(Self::Painting),
+            x if x == Self::Panda as i32 => Ok(Self::Panda),
+            x if x == Self::Parrot as i32 => Ok(Self::Parrot),
+            x if x == Self::Phantom as i32 => Ok(Self::Phantom),
+            x if x == Self::Pig as i32 => Ok(Self::Pig),
+            x if x == Self::Piglin as i32 => Ok(Self::Piglin),
+            x if x == Self::PiglinBrute as i32 => Ok(Self::PiglinBrute),
+            x if x == Self::Pillager as i32 => Ok(Self::Pillager),
+            x if x == Self::PolarBear as i32 => Ok(Self::PolarBear),
+            x if x == Self::PrimedTNT as i32 => Ok(Self::PrimedTNT),
+            x if x == Self::Pufferfish as i32 => Ok(Self::Pufferfish),
+            x if x == Self::Rabbit as i32 => Ok(Self::Rabbit),
+            x if x == Self::Ravager as i32 => Ok(Self::Ravager),
+            x if x == Self::Salmon as i32 => Ok(Self::Salmon),
+            x if x == Self::Sheep as i32 => Ok(Self::Sheep),
+            x if x == Self::Shulker as i32 => Ok(Self::Shulker),
+            x if x == Self::ShulkerBullet as i32 => Ok(Self::ShulkerBullet),
+            x if x == Self::Silverfish as i32 => Ok(Self::Silverfish),
+            x if x == Self::Skeleton as i32 => Ok(Self::Skeleton),
+            x if x == Self::SkeletonHorse as i32 => Ok(Self::SkeletonHorse),
+            x if x == Self::Slime as i32 => Ok(Self::Slime),
+            x if x == Self::SmallFireball as i32 => Ok(Self::SmallFireball),
+            x if x == Self::SnowGolem as i32 => Ok(Self::SnowGolem),
+            x if x == Self::Snowball as i32 => Ok(Self::Snowball),
+            x if x == Self::SpectralArrow as i32 => Ok(Self::SpectralArrow),
+            x if x == Self::Spider as i32 => Ok(Self::Spider),
+            x if x == Self::Squid as i32 => Ok(Self::Squid),
+            x if x == Self::Stray as i32 => Ok(Self::Stray),
+            x if x == Self::Strider as i32 => Ok(Self::Strider),
+            x if x == Self::ThrownEgg as i32 => Ok(Self::ThrownEgg),
+            x if x == Self::ThrownEnderPearl as i32 => Ok(Self::ThrownEnderPearl),
+            x if x == Self::ThrownExperienceBottle as i32 => Ok(Self::ThrownExperienceBottle),
+            x if x == Self::ThrownPotion as i32 => Ok(Self::ThrownPotion),
+            x if x == Self::ThrownTrident as i32 => Ok(Self::ThrownTrident),
+            x if x == Self::TraderLlama as i32 => Ok(Self::TraderLlama),
+            x if x == Self::TropicalFish as i32 => Ok(Self::TropicalFish),
+            x if x == Self::Turtle as i32 => Ok(Self::Turtle),
+            x if x == Self::Vex as i32 => Ok(Self::Vex),
+            x if x == Self::Villager as i32 => Ok(Self::Villager),
+            x if x == Self::Vindicator as i32 => Ok(Self::Vindicator),
+            x if x == Self::WanderingTrader as i32 => Ok(Self::WanderingTrader),
+            x if x == Self::Witch as i32 => Ok(Self::Witch),
+            x if x == Self::Wither as i32 => Ok(Self::Wither),
+            x if x == Self::WitherSkeleton as i32 => Ok(Self::WitherSkeleton),
+            x if x == Self::WitherSkull as i32 => Ok(Self::WitherSkull),
+            x if x == Self::Wolf as i32 => Ok(Self::Wolf),
+            x if x == Self::Zoglin as i32 => Ok(Self::Zoglin),
+            x if x == Self::Zombie as i32 => Ok(Self::Zombie),
+            x if x == Self::ZombieHorse as i32 => Ok(Self::ZombieHorse),
+            x if x == Self::ZombieVillager as i32 => Ok(Self::ZombieVillager),
+            x if x == Self::ZombifiedPiglin as i32 => Ok(Self::ZombifiedPiglin),
+            x if x == Self::Player as i32 => Ok(Self::Player),
+            x if x == Self::FishingHook as i32 => Ok(Self::FishingHook),
             _ => Err(Error::EnumOutOfBound)
         }
     }
